@@ -50,7 +50,7 @@
 #'   
 #' @examples
 #' ## First specificy survivals. Assume test times are 1:8, with survival function 
-#' ## at the end time 0.9  				  
+#' ## at the end time 0.9    			  
 #' surv <- exp(log(0.9)*(1:8)/8)					  
 #'
 #' ## Obtain power vs. N					  				
@@ -97,46 +97,95 @@ icpower <- function(HR, sensitivity, specificity, survivals, N = NULL, power = N
 
    	## Basic input check
    	if (!(HR>0)) stop("Check input for HR")
+   	if (!(sensitivity<=1 & sensitivity>0)) stop("Check input for sensitivity")
    	if (!(specificity<=1 & specificity>0)) stop("Check input for specificity")
    	if (sum(diff(survivals)>0)>0 | sum(survivals<0)>0 | sum(survivals>1)>0) stop("Check input for survivals")
    	if (rho<0 | rho>1) stop("Check input for rho")
    	if (alpha<0 | alpha>1) stop("Check input for alpha")
-   	if (any(pmiss < 0) | any(pmiss > 1)) stop("Check input for pmiss")
+   	if (sum(pmiss<0)>0 | sum(pmiss>1)>0) stop("Check input for pmiss")
    	if ((is.null(N)+is.null(power))!=1) stop("Need input for one and only one of power and N")
    	if (!is.null(power)) {if (any(power > 1) | any(power < 0)) stop("Check input for power")}
    	if (length(pmiss)!=1 & length(pmiss)!=length(survivals))
        stop("length of pmiss must be 1 or same length as survivals")
   	if (!(design %in% c("MCAR", "NTFP"))) stop("invalid design")
-  	if (negpred<0 | negpred>1) stop("Check input for negpred")  	
-
+  	if (negpred<0 | negpred>1) stop("Check input for negpred")
+  	
   	## Use dedicated function for perfect test, so when perfect test use alternative instead
    	if (sensitivity==1 & specificity==1) {
    		cat("For perfect test, use icpowerpf for improved computational efficiency... \n \n")
    		design <- "NTFP"
-   	}
+   		}
    	
 	miss 	<- any(pmiss!=0)  
    	beta  	<- log(HR)
    	J 		<- length(survivals)
    	Surv  	<- c(1, survivals)  	                   	
 	if (length(pmiss)==1) pmiss <- rep(pmiss, J)
-	
+
 	if (design=="MCAR" & !miss) {	
-		Dm <- powerdmat1(sensitivity, specificity, J, negpred)
-		prob <- 1	
+		## MCAR design with no missing tests
+		data 	<- as.matrix(do.call("expand.grid", rep(list(0:1), J))) 
+		o		<- 2*outer(1:J, 1:(J+1), ">=") + 1
+		smat 	<- c(specificity, 1-specificity, 1-sensitivity,  sensitivity)
+		Cm 		<- sapply(1:J, function(x) smat[outer(data[, x], o[x, ], "+")])
+		Cm 		<- apply(Cm, 1, prod)
+		Cm		<- matrix(Cm, ncol=J+1)
+		prob 	<- 1
+	
 	} else if (design=="MCAR" & miss) {
-		Dm <- powerdmat2(sensitivity, specificity, J, negpred, pmiss)
-		prob <- Dm[[2]]
-		Dm <- Dm[[1]]
+		## MCAR design with missing tests
+		data 	<- as.matrix(do.call("expand.grid", rep(list(0:2), J)))[-3^J, ]
+		o 		<- 3*outer(1:J, 1:(J+1), ">=") + 1 
+		smat 	<- c(specificity, 1-specificity, 1, 1-sensitivity, sensitivity, 1)
+		Cm	 	<- sapply(1:J, function(x) smat[outer(data[, x], o[x, ], "+")])
+		Cm	 	<- apply(Cm, 1, prod)
+		Cm	 	<- matrix(Cm, ncol=J+1)
+		prob 	<- apply(pmiss^(t(data==2))*(1-pmiss)^(t(data!=2)), 2, prod)
+
 	} else if (design=="NTFP" & !miss) {
-		Dm <- powerdmat3(sensitivity, specificity, J, negpred)
-		prob <- 1
+		## NTFP without missing tests
+		data <- sapply(0:(J-1), function(x) c(rep(0, x), 1, rep(2, J-x-1)))
+		data <- t(cbind(data, rep(0, J)))
+		o 		<- 3*outer(1:J, 1:(J+1), ">=") + 1 
+		smat <- c(specificity, 1-specificity, 1, 1-sensitivity,  sensitivity, 1)
+		Cm	 	<- sapply(1:J, function(x) smat[outer(data[, x], o[x, ], "+")])
+		Cm	 	<- apply(Cm, 1, prod)
+		Cm	 	<- matrix(Cm, ncol=J+1)
+		prob	<- 1
+			
 	} else if (design=="NTFP" & miss) {
-		Dm <- powerdmat4(sensitivity, specificity, J, negpred, pmiss)
-		prob <- Dm[[2]]
-		Dm <- Dm[[1]]
+		## NTFP with missing tests
+		smat <- c(specificity, 1-specificity, 1, 1-sensitivity,  sensitivity, 1)
+		crmat <- function(n) {
+			if (n==0) {
+				data <- matrix(1)		  
+			} else if (n==J) {
+				data <- as.matrix(do.call("expand.grid", rep(list(c(0, 2)), n)))[-2^J, ]
+			} else {
+				data <- cbind(as.matrix(do.call("expand.grid", rep(list(c(0, 2)), n))), 1)
+			}
+			n1 	<- min(n+1, J)
+			o 	<- 3*outer(1:n1, 1:(J+1), ">=") + 1
+			mats <- sapply(1:n1, function(x) smat[outer(data[, x], o[x, ], "+")])
+			mats <- apply(mats, 1, prod)
+			mats <- matrix(mats, ncol=J+1)
+			p 	<- pmiss[1:n1]
+			probs <-  apply(p^(t(data==2))*(1-p)^(t(data!=2)), 2, prod) 
+			mats <- cbind(mats, probs)
+			return(mats)
+		}
+		Cm <- do.call("rbind", sapply(0:J, crmat))
+		prob	<- Cm[, J+2]
+		Cm		<- Cm[, -(J+2)]
 	}
 
+	## Create transformation matrix and obtain D matrix
+   	mat     <- c(1, rep(0, J), rep(c(-1, 1, rep(0, J)), J-1), -1, 1)
+   	S2theta <- matrix(mat, ncol=J+1)
+   	C2C     <- negpred*diag(J+1) + (1-negpred)*matrix(c(1, rep(0,J)),
+   		       nrow=J+1, ncol=J+1)   ## Adjust for baseline misclassification  
+   	Dm      <- Cm%*%C2C%*%S2theta
+
 	I1 <- I2 <- matrix(NA, nrow=J+1, ncol=J+1)
 	
 	## Group 1 information matrix
@@ -175,4 +224,4 @@ icpower <- function(HR, sensitivity, specificity, survivals, N = NULL, power = N
         N2 <- N-N1        
         return(list(result=data.frame(N,N1,N2,power), I1=I1, I2=I2))
   	}
-}
+}	
